@@ -285,7 +285,7 @@ def _do_convert(to: Value, s: Value) -> Optional[Value]:
     if to.unstructured and s.string:
         return Value(unstructured=Unstructured(s.string.value))
 
-    if s.string or s.unstructured:
+    if s.string or s.unstructured or to.string or to.unstructured:
         return None
 
     val_s: NDArray[Any] = get_attribute(
@@ -348,9 +348,9 @@ def _strictify(s: RelaxedValue) -> Value:
     if not s:
         return Value()  # Empty list generalized into Value.empty.
     if all(isinstance(x, bool) for x in s):
-        return _strictify(Bit(s))  # type: ignore
+        return _strictify(Bit(s))
     if all(isinstance(x, (int, bool)) for x in s):
-        return _strictify(Natural64(s)) if all(x >= 0 for x in s) else _strictify(Integer64(s))  # type: ignore
+        return _strictify(Natural64(s)) if all(x >= 0 for x in s) else _strictify(Integer64(s))
     if all(isinstance(x, (float, int, bool)) for x in s):
         return _strictify(Real64(s))
 
