@@ -2,6 +2,8 @@
 # This software is distributed under the terms of the MIT License.
 # Author: Pavel Kirienko <pavel@opencyphal.org>
 
+# mypy: warn_unused_ignores=False
+
 from __future__ import annotations
 import bisect
 import asyncio
@@ -264,7 +266,7 @@ class _Matcher(typing.Generic[T]):
 
     def __init__(self, *, subject_count: int, depth: int) -> None:
         self._subject_count = int(subject_count)
-        if not self._subject_count >= 0:
+        if self._subject_count < 0:
             raise ValueError("The subject count shall be non-negative")
         self._clusters: list[_Cluster[T]] = []
         self._depth = int(depth)
@@ -273,7 +275,7 @@ class _Matcher(typing.Generic[T]):
     def update(self, key: float, tolerance: float, index: int, item: T) -> tuple[T, ...] | None:
         clust: _Cluster[T] | None = None
         # noinspection PyTypeChecker
-        ni = bisect.bisect_left(self._clusters, key)
+        ni = bisect.bisect_left(self._clusters, key)  # type: ignore
         assert 0 <= ni <= len(self._clusters)
         neigh: list[tuple[float, int]] = []
         if 0 < ni:
